/*
  statistics.cc

  Written by Taiji Yamada <taiji@aihara.co.jp>
*/
#include <iostream>
#include <vector>
#include <list>
#include <functional>
#if __cplusplus < 201103L
#include <boost/function.hpp>
#endif
#include <cstdlib>      // random(), etc
#include <cassert>
#include "statistics.hpp"
#include "statistics_online.hpp"

#define USE_LIST_FOR_MOVING_STORAGE

int main()
{
  const double maximum_permissible_error(1e-12);
  const char *result_names[] = {
    "mean",
    "variance",
    "standard deviation",
    "biased variance",
    "biased standard deviation",
    "skewness",
    "kurtosis",
    "maximum value",
    "minimum value",
    "geometric mean",
    "harmonic mean",
    "root mean square",
  };
  std::vector<double> v(/*10000*/1000);
  statistics_online<double> statistics_online;
  srandomdev();
  for (size_t i=0; i<v.size(); ++i) {
    double x((double)random()/((1L<<31)-1));
    v[i] = x;
    statistics_online.add(x);
  }
  double results[] = {
    arithmetic_average(v.begin(), v.end()),
#if 0
    unbiased_variance(v.begin(), v.end()),
    unbiased_standard_deviation(v.begin(), v.end()),
    sample_variance(v.begin(), v.end()),
    sample_standard_deviation(v.begin(), v.end()),
    skewness(v.begin(), v.end()),
    kurtosis(v.begin(), v.end()),
#else
    unbiased_variance(v.begin(), v.end(), results[0]),
    unbiased_standard_deviation(v.begin(), v.end(), results[0]),
    sample_variance(v.begin(), v.end(), results[0]),
    sample_standard_deviation(v.begin(), v.end(), results[0]),
    skewness(v.begin(), v.end(), results[0], results[2]),
    kurtosis(v.begin(), v.end(), results[0], results[2]),
#endif
    *max_element(v.begin(), v.end()),
    *min_element(v.begin(), v.end()),
    geometric_average(v.begin(), v.end()),
    harmonic_average(v.begin(), v.end()),
    generalized_average(v.begin(), v.end(), 2),
  };
  double online_results[] = {
    statistics_online.get_mean(),
    statistics_online.get_unbiased_variance(),
    statistics_online.get_unbiased_standard_deviation(),
    statistics_online.get_sample_variance(),
    statistics_online.get_sample_standard_deviation(),
    statistics_online.get_skewness(),
    statistics_online.get_kurtosis(),
    statistics_online.get_maximum_value(),
    statistics_online.get_minimum_value(),
    statistics_online.get_geometric_mean(),
    statistics_online.get_harmonic_mean(),
    statistics_online.get_root_mean_square(),
  };
  for (size_t j=0; j<sizeof(online_results)/sizeof(online_results[0]); ++j) {
    double error(sqrt(pow(online_results[j] - results[j], 2)));
    std::cout << result_names[j] << ":\t" << online_results[j] << "\t" << results[j] << "\t" << error << std::endl;
    assert(error < maximum_permissible_error);
  }

  const char *moving_result_names[] = {
    "moving arithmetic average",
    "moving geometric average",
    "moving harmonic average",
    "moving root mean square",
    "moving sample variance",
    "moving sample standard deviation",
    "moving unbiased variance",
    "moving unbiased standard deviation",
    "moving skewness",
    "moving kurtosis",
    "moving minimum value",
    "moving maximum value",
  };
#if !defined(USE_LIST_FOR_MOVING_STORAGE)
  std::vector<double> m(v.size());
#else
  std::list<double> m;
#endif
  moving_statistics_online<double> moving_statistics_online(v.size());
  for (size_t i=0; i<v.size()*10; ++i) {
    double x((double)random()/((1L<<31)-1));
#if !defined(USE_LIST_FOR_MOVING_STORAGE)
    m[i % v.size()] = x;
#else
    if (!(i < v.size())) m.pop_front();
    m.push_back(x);
#endif
    moving_statistics_online.add(x);
    if (i < v.size()) continue;
    double moving_results[] = {
      arithmetic_average(m.begin(), m.end()),
      geometric_average(m.begin(), m.end()),
      harmonic_average(m.begin(), m.end()),
      generalized_average(m.begin(), m.end(), 2),
      sample_variance(m.begin(), m.end()),
      sample_standard_deviation(m.begin(), m.end()),
      unbiased_variance(m.begin(), m.end()),
      unbiased_standard_deviation(m.begin(), m.end()),
      skewness(m.begin(), m.end()),
      kurtosis(m.begin(), m.end()),
      *min_element(m.begin(), m.end()),
      *max_element(m.begin(), m.end()),
    };
    double online_moving_results[] = {
      moving_statistics_online.get_mean(),
      moving_statistics_online.get_geometric_mean(),
      moving_statistics_online.get_harmonic_mean(),
      moving_statistics_online.get_root_mean_square(),
      moving_statistics_online.get_sample_variance(),
      moving_statistics_online.get_sample_standard_deviation(),
      moving_statistics_online.get_unbiased_variance(),
      moving_statistics_online.get_unbiased_standard_deviation(),
      moving_statistics_online.get_skewness(),
      moving_statistics_online.get_kurtosis(),
      moving_statistics_online.get_minimum_value(),
      moving_statistics_online.get_maximum_value(),
    };
    for (size_t j=0; j<sizeof(online_moving_results)/sizeof(online_moving_results[0]); ++j) {
      double error(sqrt(pow(online_moving_results[j] - moving_results[j], 2)));
      if (i+1 == v.size()*10)
        std::cout << moving_result_names[j] << ":\t" << online_moving_results[j] << "\t" << moving_results[j] << "\t" << error << std::endl;
      assert(error < maximum_permissible_error);
    }
  }
  return 0;
}
